
## This sample script is part of manuscript for the Dataset:
## Mistry, M. N. A high spatiotemporal resolution global gridded dataset of historical (1970-2018)
## human discomfort Indices based on GLDAS data. (2019). doi:10.1594/PANGAEA.904282

## The script reads Discomfort Index (DI), for year 2003.
## Script can be modified to download/read a different index (netcdf) file
## from the PANGAEA repository https://doi.org/10.1594/PANGAEA.904282

### Created by Malcolm N. Mistry, Email: malcolm.mistry@unive.it ##
### Last Update: April 14, 2020 ######

rm(list=ls(all=TRUE)) # Removes all previously created variables
gc()                  # frees up memory resources

## Check if required packages are installed ##
## If not, install and load the packages    ##

if(!require(sp)){install.packages("sp")}
if(!require(raster)){install.packages("raster")}
if(!require(latticeExtra)){install.packages("latticeExtra")}
if(!require(RCurl)){install.packages("RCurl")}

# Check current work directory 
getwd()

# Change work directory (if required)
setwd("insert_path_here") # insert full path to desired working folder, e.g.: "/home/user/Documents/HDI"

## Check Repoistory url in web browser for list of files https://doi.org/10.1594/PANGAEA.904282
## Below sample netcdf file will download to current work directory ##

url_link    <- 'http://hs.pangaea.de/model/MistryM_2019b/gldas_DI_daily_2003.nc4'

download.file(url= url_link,
              destfile='gldas_DI_daily_2003.nc4', method='curl')

## Store filename of the downloaded netcdf as a string object ##
netcdf_file <- "gldas_DI_daily_2003.nc4"

## Read the netcdf using raster brick function 
## (Package ncdf4 is another alternative way to read netcdf files)
## Consult package/function documentation for more details using '?brick'

DI_2003 <- raster::brick(netcdf_file, lvar=3, values=TRUE, level=1,
                         varname="DI") 

dim(DI_2003)        ## 600 (Lat) x 1440 (Lon) x  365 (Days or time dimension)
res(DI_2003)        ## 0.25 deg x 0.25 deg
nlayers(DI_2003)    ## Number of layers (equal to the number of Days)
names(DI_2003)      ## Displays names of each of the 365 layers (time dimension, days)

projection(DI_2003) ## Displays projection/datum

# Can also be used to set the CRS projection string (Note: This is not reprojecting data)

projection(DI_2003) <- CRS("+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0")

# Generate a date sequence for renaming layers of the raster brick object
# Note that the leading 'X' in layernames will remain because R doesn't permit 
# the column to begin with a numeral.

names(DI_2003)       <- seq.Date(from = as.Date('2003-01-01'), 
                                 to = as.Date('2003-12-31'), by = 'days')

# Generate a quick global map of DI for a particular day (in this case 07-Aug-2003)

plot(DI_2003$X2003.08.07, main=c('Discomfort Index (DI), 07-Aug-2003'))

# Aggregate to coarser resolution (here from 0.25 degree gridded to 0.5 deg gridded)
# This may be required when merging (combining) the dataframe objects 
# with another dataframe objects at 0.5 deg resolution. 

DI_2003_low_res       <- aggregate(DI_2003, fact=2) # Can take a few minutes to run

res(DI_2003_low_res)     

# Some basic raster arithematic operations.

########## (1) Compute grid-cell mean annual DI for year 2003 ###########################

# Can take a few minutes to run
DI_grid_cell_annual_mean_2003 <- raster::calc(DI_2003, function(x) mean(x,na.rm=TRUE))

# Same as below
# DI_grid_cell_annual_mean_2003 <- raster::mean(DI_2003, na.rm=TRUE) 

plot(DI_grid_cell_annual_mean_2003, main=c('Mean Discomfort Index (DI) - Year 2003'))

### (2) Compute grid-cell monthly means for each of the 12 months in year 2003 ############

# First get the date from the names of the layers and extract the month
month_index                           <- as.numeric(format(as.Date(names(DI_2003), 
                                                                   format = "X%Y.%m.%d"), format = "%m"))

# Next apply stackApply on the raster brick object with the required function (in this case 'mean')
DI_grid_cell_monthly_mean_2003        <- stackApply(DI_2003, month_index, fun = mean)

# The output will a raster brick (or a stack of 12 layers)
# Rename the layers as months (Jan:Dec)

names(DI_grid_cell_monthly_mean_2003) <- c("Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug",
                                           "Sep", "Oct", "Nov", "Dec")

##############################################################################

# Plot a panel showing mean DI for months June-August [[6:8]] represents the layers 6-8
# corresponding to the layer names 'Jun-Aug' 

plot(DI_grid_cell_monthly_mean_2003[[6:8]])

# It is often easier to work with spreadsheet type of data objects (Data frames)
# Below function coerces raster object to a dataframe, with the first two columns
# corresponding to the geo-coordinates (lon, lat) of the grid-cell, and the  subsequent
# columns (3:367) to the values (DI) of the individual days in 2003

# Since converting a large raster brick object can be memory intensive
# depending on the hardware configuration, below example first subsets
# the raster brick to the first 20 layers (i.e. 20 days) for illustration

DI_2003_df           <- as.data.frame(DI_2003[[1:20]], xy=TRUE)
# DI_2003_df           <- as.data.frame(DI_2003, xy=TRUE) # This can be memory intensive

# A number of grid-cells (over oceans for example) will have missing values since the 
# HDIs are computed only over land grid-cells. A quick way to reduce the size of the data-frame
# object in memory is by retaining only those grid-cells with valid values (or put another way
# by dropping grid-cells with missing observations)

DI_2003_df           <- DI_2003_df[complete.cases(DI_2003_df), ]

# Change column names 
columns_days         <- as.character(seq.Date(from = as.Date('2003-01-01'), 
                                              to = as.Date('2003-01-20'), by = 'days'))

# columns_days         <- as.character(seq.Date(from = as.Date('2003-01-01'), 
#                                      to = as.Date('2003-12-31'), by = 'days'))

names(DI_2003_df)    <- c('Lon', 'Lat', columns_days)

colnames(DI_2003_df) # Display column names

## Using raster brick object 'DI_2003' and a shape file for Italy, extract DI for spatial
## boundaries (here regional boundaries for Italy). Country specific shape files can be found
## here https://gadm.org/data.html . Level 1 corresponds to regions. Level 2 corresponds to provinces.

### Download the shapes (actually spatial polygon data frames) in loop for the current country ###

shapefile_url          <- 'https://biogeo.ucdavis.edu/data/gadm3.6/Rsp/gadm36_ITA_1_sp.rds'

## The shape file will downloaded and saved as a SpatialPolygonsDataFrame(SPDF) object ##

Italy_spdf             <- readRDS(url(shapefile_url))

# colnames(Italy_spdf@data) # NAME_1 column is regions

# Prior any spatial extraction involving two objects (raster and spdf or two spdfs etc), 
# the two objects should be on the same CRS and have the same projection

isTRUE(proj4string(Italy_spdf)==proj4string(DI_2003))      ## Will (should) return TRUE

# Now crop and mask the large (global) raster brick object to Italy country level boundary 
# Here grid_cell_monthly_mean_2003 is used an example, but the same steps apply for the daily
# or annual DI raster objects in above steps

DI_grid_cell_monthly_mean_2003_Italy_cropped    <- raster::crop(DI_grid_cell_monthly_mean_2003, extent(Italy_spdf)) 
DI_grid_cell_monthly_mean_2003_Italy_rasterized <- raster::rasterize(Italy_spdf, DI_grid_cell_monthly_mean_2003_Italy_cropped) # Not necessary
DI_grid_cell_monthly_mean_2003_Italy            <- raster::mask(x=DI_grid_cell_monthly_mean_2003_Italy_cropped, mask=DI_grid_cell_monthly_mean_2003_Italy_rasterized)

### Better plots (maps) using package 'sp'. Whitespace (margin) is removed.
## See package documentation for additional options ##

spplot(DI_grid_cell_monthly_mean_2003_Italy$Aug,  main=list(label="Average DI (Aug-2003) Italy", cex=1.0))+ 
       latticeExtra::layer(sp.polygons(Italy_spdf))             # Overlay Italy regional boundaries

# Spatially aggregate the monthly DI to the regional level (Italy) and convert to dataframe
# This will involve aggregation (computing spatial mean) of the raster pixels (grid-cells) falling
# within the regional boundaries (polygons). 

# Note that this is done for each month, i.e. the number of layers (12) in 
# the raster brick object are maintained.  

DI_monthly_mean_2003_Italy_regions_df   <- raster::extract(DI_grid_cell_monthly_mean_2003_Italy, Italy_spdf, 
                                                           fun=mean, na.rm=TRUE, df=TRUE)

# Rename the columns in the DF and merge the column 'region name' from SPDF object (This is the fourth column in SPDF object)
# colnames(Italy_spdf@data) 
# head(Italy_spdf@data)     # Disply first few rows

DI_monthly_mean_2003_Italy_regions_df        <-  cbind.data.frame(Italy_spdf@data[c(4)], 
                                                                  DI_monthly_mean_2003_Italy_regions_df[2:13])

names(DI_monthly_mean_2003_Italy_regions_df) <- c("Region", "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug",
                                                  "Sep", "Oct", "Nov", "Dec")

# Plot and save the monthly DI at grid-cell level over Italy as a .png file #
# .png file will save in current work directory

png("mean_Aug_2003_DI_grid_cell_level_Italy.png")

plot(DI_grid_cell_monthly_mean_2003_Italy$Aug, 
     main="Average DI (Aug-2003) Italy",
     axes=TRUE, xlab="Lon", ylab="Lat", col=topo.colors(100), 
     legend.args=list(text='Degree C', side=4, font=2, line=2.5, cex=0.8))

plot(Italy_spdf, add=TRUE)  ## Overlap boundary of Italy over the pixels

dev.off()                   ## Close png file

# Save monthly mean DI (grid-cell level and aggregated to Italian regions) dataframe object as .csv file.
## Will save in current workspace

write.csv(DI_monthly_mean_2003_Italy_regions_df, 
          file="DI_monthly_mean_2003_Italy_regions.csv",
          row.names = FALSE)

## Save the raster brick as Geotiff file ####

writeRaster(DI_grid_cell_monthly_mean_2003_Italy, "DI_grid_cell_monthly_mean_2003_Italy.tif",
            format = "GTiff", overwrite=TRUE, NAflag=-9999)                 ## set no data value to -9999. 

## Save the raster brick as netcdf file ####

writeRaster(DI_grid_cell_monthly_mean_2003_Italy, "DI_grid_cell_monthly_mean_2003_Italy.tif",
            format = "CDF", overwrite=TRUE, varname="DI", varunit="degree C", 
            longname="Discomfort Index (DI)", xname="lon",   yname="lat", zname="time")        

